#include <sys/cdefs.h>
__FBSDID("$FreeBSD$");

/* write and check read a file */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <err.h>

#include "stress.h"

static char path[128];
static int starting_dir;
static unsigned long size;

#define MAXSIZE 256 * 1024 * 1024

int
setup(int nb)
{
	unsigned long df;
	FILE *fd;

	umask(0);
	sprintf(path,"%s.%05d", getprogname(), getpid());
	(void)mkdir(path, 0770);
	if (chdir(path) == -1)
		err(1, "chdir(%s), %s:%d", path, __FILE__, __LINE__);
	if ((starting_dir = open(".", 0)) < 0)
		err(1, ".");

	if ((fd = popen("df -k . | tail -1 | awk '{print $4}'", "r")) == NULL)
		err(1, "popen");
	if (fscanf(fd, "%ld", &df) != 1)
		err(1, "fscanf() %s:%d", __FILE__, __LINE__);
	if (pclose(fd) == -1)
		err(1, "pclose");

	if (op->verbose > 2)
		printf("df -k : %ld\n", df);
	df = df * 1024;
	if (df > MAXSIZE)
		df = MAXSIZE;

	if (op->hog == 0)
		size = random_int((int)(df * 0.1), (int)(df * 0.8));
	size = size / op->incarnations;

	if (op->verbose > 1)
		printf("Testing rw with %ldMb, pid %d\n", size / 1024 / 1024,
				getpid());

	size = size / sizeof(int);

	return (0);
}

void
cleanup(void)
{
	if (fchdir(starting_dir) == -1)
		err(1, "fchdir()");

	(void)system("find . -delete");

	if (chdir("..") == -1)
		err(1, "chdir(..)");
	if (rmdir(path) == -1)
		err(1, "rmdir(%s), %s:%d", path, __FILE__, __LINE__);
}

int
test(void)
{
	int buf[1024], index, to;
#ifdef TEST
	int i;
#endif
	int fd;
	char file[128];


	sprintf(file,"p%05d", getpid());
	if ((fd = creat(file, 0660)) == -1) 
		err(1, "creat(%s)", file);

	index = 0;
	while (index < size && done_testing == 0) {
		to = sizeof(buf)/sizeof(int);
		if (to >= size) to = size;
#ifdef TEST
		for (i = 0; i < to; i++)
			buf[i] = index + i;
#endif
		index += to;
		if (write(fd, buf, to * sizeof(int)) != sizeof(buf))
			err(1, "write(%s), %s:%d", file, __FILE__, __LINE__);
	}
	if (close(fd) == -1)
		err(1, "close(%s), %s:%d", file, __FILE__, __LINE__);
	if ((fd = open(file, O_RDONLY)) == -1)
		err(1, "open(%s), %s:%d", file, __FILE__, __LINE__);

	index = 0;
	while (index < size && done_testing == 0) {
		to = sizeof(buf)/sizeof(int);
		if (to >= size) to = size;
		if (read(fd, buf, to * sizeof(int)) != sizeof(buf))
			perror("rw read");
#ifdef TEST
		for (i = 0; i < to; i++) {
			if (buf[i] != index + i) {
				fprintf(stderr,
					"%s, pid %d: expected %d @ %d, got %d\n",
					getprogname(), getpid(), index+i, index+i,
					buf[i]);
				exit(EXIT_FAILURE);
			}
		}
#endif
		index += to;
	}
	if (close(fd) == -1)
		err(1, "close(%s), %s:%d", file, __FILE__, __LINE__);
	if (unlink(file) == -1)
		err(1, "unlink(%s), %s:%d", file, __FILE__, __LINE__);
	return (0);
}
